<?php

namespace Teknisa\MAN\Factories\Retail;

use Doctrine\DBAL\Connection;

class TabelaPreco extends BaseRepo {
    private $productFactory;

    public function __construct(Connection $conn, Produto $productFactory) {
        parent::__construct($conn);
        $this->productFactory = $productFactory;
    }
    
    /**
     * $pricingTableFilters must be an associative array having:
     * - CDTABEPREC
     * - DTINIVIGENCIA
     * - CDFILIAL
     * - NRORG
     * 
     * $productFilters must be an associative array having:
     * - PRODUCTS (A string with CDPRODUTO's joined by '_'. Example: 1001001001_90771239720_
     * - CDARVPRODINI (A string with the initial CDARVPROD).
     * - CDARVPRODFIN (A string with the final CDARVPROD).
     */
    
    public function getProductsPrice(array $pricingTableFilters, array $productFilters) {
        $CDGRUPOPROD = $this->getCDGRUPOPROD($pricingTableFilters);
        $CDSEGMER    = $this->getCDSEGMER($pricingTableFilters);
        
        if(empty($CDGRUPOPROD)) {
            if(empty($CDSEGMER)) {
                // Não utiliza segmento de mercado, nem família
                $CDNVPRODUTO = $this->productFactory->getCDNVPRODUTO($NRORG);
                return $this->getProductsNoSegNoFam($pricingTableFilters, $productFilters, $CDNVPRODUTO);
            } else {
                // Utiliza somente segmento de mercado
                return $this->getProductsSeg($pricingTableFilters, $productFilters, $CDSEGMER);
            }
        } else {
            $CDFILISUBGR = $this->getCDFILISUBGR($pricingTableFilters, $CDGRUPOPROD);
            if(empty($CDSEGMER)) {
                // Utiliza somente família
                return $this->getProductsFam($pricingTableFilters, $productFilters, $CDFILISUBGR);
            } else {
                // Utiliza família e segmento de mercado
                return $this->getProductsSegFam($pricingTableFilters, $productFilters, $CDFILISUBGR, $CDSEGMER);
            }
        }
    }

    private function getCDSEGMER($pricingTableFilters) {
        $params = array(
            ':CDFILIAL' => $pricingTableFilters['CDFILIAL'],
            ':NRORG'    => $pricingTableFilters['NRORG']
        );
        
        $result = $this->conn->fetchAll(self::GET_CDSEGMER, $params);
        
        return empty($result) ? null : $result[0]['CDSEGMER'];
    }

    private function getCDGRUPOPROD($pricingTableFilters) {
        $params = array(
            ':CDFILIAL' =>  $pricingTableFilters['CDFILIAL'],
            ':NRORG'    =>  $pricingTableFilters['NRORG']
        );
        
        $result = $this->conn->fetchAll(self::GET_CDGRUPPROD, $params);
        
        return empty($result) ? null : $result[0]['CDGRPRODPDV'];
    }
    
    private function getCDFILISUBGR ($pricingTableFilters, $CDGRUPOPROD) {
        return $this->getNRCONFIG($pricingTableFilters, $CDGRUPOPROD) > 0 
                        ? $pricingTableFilters['CDFILIAL']
                        : 'P';
    }
    
    private function getNRCONFIG($pricingTableFilters, $CDGRUPOPROD) {
        $params = array(
            ':CDFILISUBGR'    =>  $pricingTableFilters['CDFILIAL'],
            ':NRORG'          =>  $pricingTableFilters['NRORG'],
            ':CDGRUPPROD'     =>  $CDGRUPPROD
        );
        
        $result = $this->conn->fetchAll(self::GET_NRCONFIG, $params);
        
        return empty($result) ? null : $result[0]['NRCONFIG'];
    }
   
    private function getProductsSegFam($pricingTableFilters, $productsFilter,  $CDFILISUBGR, $CDSEGMER) {
        $params = array(
            ':PRODUTOS'       =>  $productsFilter['PRODUTOS'],
            ':CDARVPRODINI'   =>  $productsFilter['CDARVPRODINI'],
            ':CDARVPRODFIN'   =>  $productsFilter['CDARVPRODFIN'],
            
            ':CDFILIAL'       =>  $pricingTableFilters['CDFILIAL'],
            ':CDTABEPREC'     =>  $pricingTableFilters['CDTABEPREC'],
            ':DTINIVGPREC'    =>  $pricingTableFilters['DTINIVGPREC'],
            ':CDFILISUBGR'    =>  $CDFILISUBGR,
            ':CDSEGMER'       =>  $CDSEGMER,
            ':NRORG'          =>  $pricingTableFilters['NRORG']
        );
        
        $result = $this->conn->fetchAll(self::GET_PRODUCTS_SEG_FAM, $params);
        
        return empty($result) ? null : $result[0]['NRCONFIG'];
    } 
    
    private function getProductsFam($pricingTableFilters, $productsFilter, $CDFILISUBGR) {
        $params = array(
            ':PRODUTOS'       =>  $productsFilter['PRODUTOS'],
            ':CDARVPRODINI'   =>  $productsFilter['CDARVPRODINI'],
            ':CDARVPRODFIN'   =>  $productsFilter['CDARVPRODFIN'],
            
            ':CDFILIAL'       =>  $pricingTableFilters['CDFILIAL'],
            ':CDTABEPREC'     =>  $pricingTableFilters['CDTABEPREC'],
            ':DTINIVGPREC'    =>  $pricingTableFilters['DTINIVGPREC'],
            ':CDFILISUBGR'    =>  $CDFILISUBGR,
            ':NRORG'          =>  $pricingTableFilters['NRORG']
        );

        return $this->conn->fetchAll(self::GET_PRODUCTS_FAM, $params);
    } 
    
    private function getProductsSeg($pricingTableFilters, $productsFilter, $CDSEGMER) {
        $params = array(
            ':PRODUTOS'       =>  $productsFilter['PRODUTOS'],
            ':CDARVPRODINI'   =>  $productsFilter['CDARVPRODINI'],
            ':CDARVPRODFIN'   =>  $productsFilter['CDARVPRODFIN'],
            
            ':CDFILIAL'       =>  $pricingTableFilters['CDFILIAL'],
            ':CDTABEPREC'     =>  $pricingTableFilters['CDTABEPREC'],
            ':DTINIVGPREC'    =>  $pricingTableFilters['DTINIVGPREC'],
            ':CDSEGMER'       =>  $CDSEGMER,
            ':NRORG'          =>  $pricingTableFilters['NRORG']
        );
        
        return $this->conn->fetchAll(self::GET_PRODUCTS_SEG, $params);
    } 
    
    private function getProductsNoSegNoFam($pricingTableFilters, $productsFilter) {
        $params = array(
            ':PRODUTOS'       =>  $productsFilter['PRODUTOS'],
            ':CDARVPRODINI'   =>  $productsFilter['CDARVPRODINI'],
            ':CDARVPRODFIN'   =>  $productsFilter['CDARVPRODFIN'],
            
            ':CDFILIAL'       =>  $pricingTableFilters['CDFILIAL'],
            ':CDTABEPREC'     =>  $pricingTableFilters['CDTABEPREC'],
            ':DTINIVGPREC'    =>  $pricingTableFilters['DTINIVGPREC'],
            ':NRORG'          =>  $pricingTableFilters['NRORG']
        );
        
        return $this->conn->fetchAll(self::GET_PRODUCTS, $params);
    } 
    
    public function updateMarkupCols (
                        $pricingTableFilters,
                        $CDPRODUTO,
                        $VRCUSTITEM,
                        $VRMKUPITEM
                    ) {
        $params = array(
            ':VRCUSTITEM'     =>  $VRCUSTITEM,
            ':VRMKUPITEM'     =>  $VRMKUPITEM,
            
            ':CDFILIAL'       =>  $pricingTableFilters['CDFILIAL'],
            ':CDTABEPREC'     =>  $pricingTableFilters['CDTABEPREC'],
            ':DTINIVGPREC'    =>  $pricingTableFilters['DTINIVGPREC'],
            ':NRORG'          =>  $pricingTableFilters['NRORG']
        );
        
        return $this->conn->executeQuery(self::UPDATE_ITEMPRECO_MARKUP_COLS, $params);                        
    }
    // Sem Segmento de mercado e sem Família
    const GET_PRODUCTS = "
        SELECT
            IP.CDFILIAL,
            IP.CDTABEPREC,
            IP.HRINIVENPROD,
            IP.HRFIMVENPROD,
            PR.CDNVPRODUTO,
            PR.CDPRODUTO,
            NVL(IP.CDPRPAITABPR, PR.CDPRODUTO) AS CDPRPAITABPR,
            IP.VRPRECITEM,
            NVL(IP.VRPRECITEMCL, 0) AS VRPRECITEMCL, IP.DTINIVGPREC,
            PR.NMPRODUTO,
            NVL(NVL(R.NRCOMPPROD, 0), 0) AS NRCOMPPROD,
            IP.IDPRECVARIA, PR.CDARVPROD,
            NVL(IP.VRPRESUGITEM,0) AS VRPRESUGITEM,
            NVL(VRCUSTITEM, 0) AS VRCUSTITEM, NVL(VRMKUPITEM, 0) AS VRMKUPITEM,
            IP.IDPRODPROMOC,
            CDBARPRODUTO,
            IP.NRORG
        FROM
            PRODUTO                                                                                             PR,
            ITEMPRECO                                                                                           IP,
            (
                SELECT
                    COUNT(*) AS NRCOMPPROD,
                    CDPRODUTO,
                    NRORG
                FROM
                    ROTFABRICA
                WHERE
                    NRORG = :NRORG
                GROUP BY
                    CDPRODUTO,
                    NRORG
            ) R
        WHERE
            ( PR.CDNVPRODUTO = :CDNVPRODUTO )
            AND ( PR.NRORG = :NRORG )
            AND ( PR.CDPRODUTO = IP.CDPRODUTO )
            AND ( INSTR(:PRODUTOS, PR.CDARVPROD) <> 0 OR (PR.CDARVPROD BETWEEN :CDARVPRODINI AND :CDARVPRODFIN) )
            AND ( PR.NRORG = IP.NRORG )
            AND ( PR.CDPRODUTO = NVL(IP.CDPRPAITABPR, PR.CDPRODUTO) )
            AND ( IP.CDPRPAITABPR = PR.CDPRODUTO )
            AND ( IP.CDFILIAL = :CDFILIAL )
            AND ( IP.CDTABEPREC = :CDTABEPREC )
            AND ( IP.DTINIVGPREC = TRUNC(TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH24:MI:SS')) )
            AND ( R.CDPRODUTO (+) = PR.CDPRODUTO )
            AND ( R.NRORG (+) = PR.NRORG )
            AND ( NVL(PR.IDPRODATIVPR, 'S') = 'S' )
        ORDER BY
            PR.CDARVPROD
    ";
    
    const GET_PRODUCTS_SEG = "
        SELECT
            ' ' AS CDFILISUBGR,
            IP.CDTABEPREC,
            IP.HRINIVENPROD,
            IP.HRFIMVENPROD,
            IP.CDFILIAL,
            PR.CDNVPRODUTO,
            PR.NRORG,
            PR.CDPRODUTO,
            NVL(IP.CDPRPAITABPR, PR.CDPRODUTO) AS CDPRPAITABPR,
            IP.VRPRECITEM,
            IP.VRPRECITEMCL,
            IP.DTINIVGPREC,
            PR.NMPRODUTO,
            NVL(NVL(R.NRCOMPPROD, 0), 0) AS NRCOMPPROD,
            IP.IDPRECVARIA, PR.CDARVPROD,
            NVL(IP.VRPRESUGITEM, 0) AS VRPRESUGITEM,
            NVL(VRCUSTITEM, 0) AS VRCUSTITEM, VRMKUPITEM,
            IP.IDPRODPROMOC,
            CDBARPRODUTO,
            S.CDSEGMER
        FROM
            FILIAL                                                                                              FI,
            PRODUTOSEG                                                                                          S,
            PRODUTO                                                                                             PR,
            ITEMPRECO                                                                                           IP,
            (
                SELECT
                    COUNT(*) AS NRCOMPPROD,
                    CDPRODUTO,
                    NRORG
                FROM
                    ROTFABRICA
                WHERE
                    NRORG = :NRORG
                GROUP BY
                    CDPRODUTO,
                    NRORG
            ) R
        WHERE ( FI.CDFILIAL = IP.CDFILIAL )
            AND ( S.CDSEGMER = FI.CDSEGMER )
            AND ( PR.CDPRODUTO = S.CDPRODUTO )
            AND ( PR.NRORG = S.NRORG )
            AND ( INSTR(:PRODUTOS, PR.CDARVPROD) <> 0 OR (PR.CDARVPROD BETWEEN :CDARVPRODINI AND :CDARVPRODFIN) )
            AND ( PR.CDPRODUTO = NVL(IP.CDPRPAITABPR, PR.CDPRODUTO) )
            AND ( IP.CDPRPAITABPR = PR.CDPRODUTO )
            AND ( IP.CDPRODUTO = PR.CDPRODUTO )
            AND ( IP.NRORG = PR.NRORG )
            AND ( IP.CDFILIAL = :CDFILIAL )
            AND ( IP.CDTABEPREC = :CDTABEPREC )
            AND ( IP.DTINIVGPREC = TRUNC(TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH24:MI:SS')) )
            AND ( R.CDPRODUTO (+) = PR.CDPRODUTO )
            AND ( R.NRORG (+) = PR.NRORG )
            AND ( NVL(PR.IDPRODATIVPR, 'S') = 'S' )
            AND ( PR.NRORG = :NRORG )
        ORDER BY
            PR.CDARVPROD
    ";
    
    const GET_PRODUCTS_FAM = "
        SELECT DISTINCT
            PV.CDFILIAL,
            IP.HRINIVENPROD,
            IP.HRFIMVENPROD,
            IP.CDTABEPREC,
            PS.CDFILISUBGR,
            PR.NRORG,
            PR.CDPRODUTO,
            NVL(IP.CDPRPAITABPR, PR.CDPRODUTO) AS CDPRPAITABPR,
            IP.VRPRECITEM,
            NVL(IP.VRPRECITEMCL,0) AS VRPRECITEMCL,
            IP.DTINIVGPREC,
            PR.CDNVPRODUTO,
            PR.NMPRODUTO, IP.IDPRECVARIA, PR.CDARVPROD,
            NVL(IP.VRPRESUGITEM,0) AS VRPRESUGITEM,
            NVL(VRCUSTITEM,0) AS VRCUSTITEM,
            NVL(VRMKUPITEM,0) AS VRMKUPITEM,
            IP.IDPRODPROMOC,
            CDBARPRODUTO
        FROM
            ITEMPRECO     IP
            JOIN PRODUTO       PR ON IP.CDPRODUTO = PR.CDPRODUTO
                               AND IP.NRORG = PR.NRORG
            LEFT JOIN PRSUBGRPROD   PS ON ( PR.CDPRODUTO = PS.CDPRODUTO )
                                        AND ( PR.NRORG = PS.NRORG )
            LEFT JOIN PARAVEND      PV ON PS.CDGRUPPROD = PV.CDGRUPPROD
                                     AND IP.CDFILIAL = PV.CDFILIAL
                                     AND IP.NRORG = PV.NRORG
        WHERE
            ( PV.CDFILIAL = :CDFILIAL )
            AND ( ( PS.CDFILISUBGR = :CDFILIAL ) OR ( PS.CDFILISUBGR = 'P' ) )
            AND ( INSTR(:PRODUTOS, PR.CDARVPROD) <> 0 OR (PR.CDARVPROD BETWEEN :CDARVPRODINI AND :CDARVPRODFIN) )
            AND ( IP.CDFILIAL = :CDFILIAL )
            AND ( IP.CDTABEPREC = :CDTABEPREC )
            AND ( IP.DTINIVGPREC = TRUNC(TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH24:MI:SS')) )
            AND (NVL(PR.IDPRODATIVPR, 'S') = 'S' )
            AND ( PR.NRORG = :NRORG )
        ORDER BY
            PR.CDARVPROD
    ";
    
    const GET_PRODUCTS_SEG_FAM = "
        SELECT DISTINCT
            ' ' AS CDFILISUBGR,
            IP.HRINIVENPROD,
            IP.HRFIMVENPROD,
            PR.CDNVPRODUTO,
            PR.NRORG,
            PR.CDPRODUTO,
            NVL(IP.CDPRPAITABPR, PR.CDPRODUTO) AS CDPRPAITABPR,
            IP.CDFILIAL,
            IP.CDTABEPREC,
            IP.VRPRECITEM,
            NVL(IP.VRPRECITEMCL, 0) AS VRPRECITEMCL,
            IP.DTINIVGPREC,
            PR.NMPRODUTO, NVL(NVL(R.NRCOMPPROD, 0) + NVL(F.NRCOMPPRODL, 0), 0) AS NRCOMPPROD, ' ' AS CDSEGMER,
            IP.IDPRECVARIA,
            PR.CDARVPROD,
            NVL(IP.VRPRESUGITEM, 0) AS VRPRESUGITEM,
            NVL(VRCUSTITEM, 0) AS VRCUSTITEM,
            VRMKUPITEM,
            IP.IDPRODPROMOC,
            CDBARPRODUTO
        FROM
            FILIAL                                                                                                                         FI, 
            PRODUTOSEG                                                                                                                     PG,
            PRSUBGRPROD                                                                                                                    PS,
            PARAVEND                                                                                                                       PV,
            PRODUTO                                                                                                                        PR,
            ITEMPRECO                                                                                                                      IP,
            (
                SELECT
                    COUNT(*) AS NRCOMPPROD,
                    CDPRODUTO,
                    NRORG
                FROM
                    ROTFABRICA
                WHERE
                    NRORG = :NRORG
                GROUP BY
                    CDPRODUTO,
                    NRORG
            )              R,
            (
                SELECT
                    COUNT(*) AS NRCOMPPRODL,
                    CDPRODUTO,
                    NRORG
                FROM
                    ROTFABLOCAL
                WHERE
                    ( CDFILIAL = :CDFILIAL )
                    AND ( NRORG = :NRORG )
                GROUP BY
                    CDPRODUTO,
                    NRORG
            ) F
        WHERE ( FI.CDSEGMER = PG.CDSEGMER )
            AND ( PV.CDFILIAL = FI.CDFILIAL )
            AND ( PV.CDFILIAL = :CDFILIAL )
            AND ( PS.CDGRUPPROD = PV.CDGRPRODPDV )
            AND ( PS.NRORG = PV.NRORG )
            AND ( INSTR(:PRODUTOS, PR.CDARVPROD) <> 0 OR (PR.CDARVPROD BETWEEN :CDARVPRODINI AND :CDARVPRODFIN) )
            AND ( ( PS.CDFILISUBGR = :CDFILIAL ) OR ( :CDFILISUBGR = 'P' ) )
            AND ( PS.CDPRODUTO = PG.CDPRODUTO )
            AND ( PS.NRORG = PG.NRORG )
            AND ( PR.CDPRODUTO = NVL(IP.CDPRPAITABPR, PR.CDPRODUTO) )
            AND ( PR.CDPRODUTO = PS.CDPRODUTO )
            AND ( PR.NRORG = PS.NRORG )
            AND ( IP.CDPRPAITABPR = PR.CDPRODUTO )
            AND ( IP.CDPRODUTO = PR.CDPRODUTO )
            AND ( IP.NRORG = PR.NRORG )
            AND ( IP.CDFILIAL = :CDFILIAL )
            AND ( IP.CDTABEPREC = :CDTABEPREC )
            AND ( IP.DTINIVGPREC = TRUNC(TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH24:MI:SS')) )
            AND ( R.CDPRODUTO (+) = PR.CDPRODUTO )
            AND ( R.NRORG (+) = PR.NRORG )
            AND ( F.CDPRODUTO (+) = PR.CDPRODUTO )
            AND ( F.NRORG (+) = PR.NRORG )
            AND ( NVL(PR.IDPRODATIVPR, 'S') = 'S' )
            AND ( PR.NRORG = :NRORG )
        ORDER BY
            PR.CDARVPROD
    ";
    
    const GET_CDGRUPPROD = "
  	SELECT
     PV.CDGRPRODPDV,
     GP.CDGRUPPROD||' - '||GP.NMGRUPPROD AS GRUPOPROD
    FROM
       PARAVEND PV, GRUPPROD GP
    WHERE
       (PV.CDFILIAL    = :CDFILIAL  ) AND
       (PV.NRORG       = :NRORG) AND
       (PV.NRORG       = GP.NRORG) AND
       (PV.CDGRPRODPDV = GP.CDGRUPPROD)
	";
	
	const GET_NRCONFIG = "
    	SELECT NRCONFIG
	    	FROM
	    	(SELECT
	       		COUNT(*) AS NRCONFIG
	    	  FROM PRSUBGRPROD
		    WHERE
		       (CDFILISUBGR = :CDFILISUBGR) AND
		       (CDGRUPPROD  = :CDGRUPPROD ) AND
		       (NRORG       = :NRORG)
		   	)
    ";
     
    const GET_CDSEGMER = "
		SELECT DISTINCT
		   FI.CDFILIAL  , FI.NMFILIAL, FI.CDSEGMER, FI.CDEMPRESA,
		   FI.CDINSCESTA, E.CDPAIS   , E.SGESTADO , E.CDMUNICIPIO,
		   SE.CDSEGMER||' - '||SE.DSSEGMER AS SEGMENTO,
		   PA.IDPRECDIFCOM
		FROM
		   FILIAL FI
       LEFT JOIN SEGMERCADO SE ON
        (FI.CDSEGMER = SE.CDSEGMER)AND
        (FI.NRORG    = SE.NRORG)
      LEFT JOIN PARAVEND PA ON
        (FI.CDFILIAL = PA.CDFILIAL) AND
        (FI.NRORG    = PA.NRORG)
      LEFT JOIN ENDEFILI E ON
        (FI.CDFILIAL = E.CDFILIAL) AND
        (FI.NRORG    = E.NRORG   )
		WHERE
		  (FI.CDFILIAL = :CDFILIAL) AND
      (FI.NRORG    = :NRORG   )		  
		ORDER BY
		   FI.CDFILIAL
	";
	
	const UPDATE_ITEMPRECO_MARKUP_COLS = "
	    UPDATE ITEMPRECO
           SET VRCUSTITEM  = :VRCUSTITEM,
               VRMKUPITEM  = :VRMKUPITEM
         WHERE CDFILIAL    = :CDFILIAL 
           AND CDTABEPREC  = :CDTABEPREC 
           AND DTINIVGPREC = TRUNC(TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH24:MI:SS'))
           AND NRORG       = :NRORG
	";
}
